package com.example.springboot.service.impl;

import com.example.springboot.entity.Auth.Account;
import com.example.springboot.mapper.UserMapper;
import com.example.springboot.service.AuthorizeService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.mail.MailException;
import org.springframework.mail.MailSender;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Optional;
import java.util.Random;
import java.util.concurrent.TimeUnit;

@Service
public class AuthorizeServiceImpl implements AuthorizeService {
    @Value("${spring.mail.username}")
    String from;

    @Resource
    UserMapper userMapper;

    @Resource
    MailSender mailSender;

    @Resource
    StringRedisTemplate template;

    BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        if (username == null){
            throw new UsernameNotFoundException("用户名不能为空");
        }
        Account accountByNameOrEmail = userMapper.findAccountByNameOrEmail(username);
        if (accountByNameOrEmail == null){
            throw new UsernameNotFoundException("用户名或密码错误");
        }
        return User
                .withUsername(accountByNameOrEmail.getUsername())
                .password(accountByNameOrEmail.getPassword())
                .roles("user")
                .build();
    }

    /*
     * 1、先生成对应的验证码
     * 2、把邮箱和对应的验证码直接存放到Redis里（验证码过期时间为3分钟，如果想要重新发送验证码，需判断验证码剩余时间小于2分钟，才可重新发送）
     * 3、发送验证码到指定邮箱
     * 4、如果发送失败，需把Redis里刚刚插入的数据删除
     * 5、用户在注册时，再从Redis中取出对应的键值对，然后判断验证码是否一致
     * */
    @Override
    public String sendValidateEmail(String email, String sessionId, boolean hasAccount) {
        String key = "email:" + sessionId + ":" + email + ":" + hasAccount;
        System.out.println("key:" + key);
        if (Boolean.TRUE.equals(template.hasKey(key))){
            Long expire = Optional.ofNullable(template.getExpire(key, TimeUnit.SECONDS)).orElse(0L);
            if (expire > 120){
                return "请求频繁，请稍后再试";
            }
        }
        Account dbUser = userMapper.findAccountByNameOrEmail(email);
        if (hasAccount && dbUser == null){
            return "没有此邮件地址的账户";
        }
        if (!hasAccount && dbUser != null){
            return "此邮箱已注册用户，请更换其它邮箱";
        }
        Random random = new Random();
        int code = random.nextInt(899999) + 100000;
        SimpleMailMessage message = new SimpleMailMessage();
        message.setFrom(from);
        message.setTo(email);
        message.setSubject("这是您注册程序所需的验证码邮件");//邮件标题
        message.setText("验证码为："+code);
        System.out.println("验证码为："+code);
        try {
            mailSender.send(message);
            template.opsForValue().set(key,String.valueOf(code), 3, TimeUnit.MINUTES);
            return null;
        } catch (MailException e) {
            e.getMessage();
            return "邮件发送失败，请检查邮箱地址是否正确";
        }
    }

    @Override
    public String vaildateAndRegister(String username, String password, String email, String code, String sessionId) {
        String key = "email:" + sessionId + ":" + email + ":false";
        if (Boolean.TRUE.equals(template.hasKey(key))) {
            String s = template.opsForValue().get(key);
            if (s == null){
                return "验证码失效，请重新获取";
            }
            if (s.equals(code)){
                Account dbUser = userMapper.findAccountByNameOrEmail(username);
                if (dbUser != null){
                    return "用户名已被占用，请更换其它用户名！";
                }
                template.delete(key);
                password = encoder.encode(password);
                if (userMapper.createAccount(username, password, email) > 0){
                    return null;
                } else {
                    return "内部错误，请联系管理员";
                }
            } else {
                return "验证码错误，请核实后再提交";
            }
        } else {
            return "请先获取验证码";
        }

    }

    @Override
    public String vaildateOnly(String email, String code, String sessionId) {
        String key = "email:" + sessionId + ":" + email + ":true";
        if (Boolean.TRUE.equals(template.hasKey(key))) {
            String s = template.opsForValue().get(key);
            if (s == null){
                return "验证码失效，请重新获取";
            }
            if (s.equals(code)){
                template.delete(key);
                return null;
            } else {
                return "验证码错误，请核实后再提交";
            }
        } else {
            return "请先获取验证码";
        }
    }

    @Override
    public boolean resetPassword(String password, String email) {
        password = encoder.encode(password);
        return userMapper.resetPassword(password,email) > 0;
    }
}
